DefinitionBlock ("SSDT-DEBUG", "SSDT", 2, "hack", "debug", 0)
{
Device (RMDT)
{
    Name (_HID, "RMD0000")
    Name (RING, Package(256) { })
    Mutex (RTMX, 0)
    Name (HEAD, 0)
    Name (TAIL, 0)
    // PUSH: Use to push a trace item into RING for ACPIDebug.kext
    Method (PUSH, 1, NotSerialized)
    {
        Acquire(RTMX, 0xFFFF)
        // push new item at HEAD
        Add(HEAD, 1, Local0)
        If (LGreaterEqual(Local0, SizeOf(RING))) { Store(0, Local0) }
        if (LNotEqual(Local0, TAIL))
        {
            Store(Arg0, Index(RING, HEAD))
            Store(Local0, HEAD)
        }
        Release(RTMX)
        Notify(RMDT, 0x80)
    }
    // FTCH: Used by ACPIDebug.kext to fetch an item from RING
    Method (FTCH, 0, NotSerialized)
    {
        Acquire(RTMX, 0xFFFF)
        // pull item from TAIL and return it
        Store(0, Local0)
        if (LNotEqual(HEAD, TAIL))
        {
            Store(DerefOf(Index(RING, TAIL)), Local0)
            Increment(TAIL)
            If (LGreaterEqual(TAIL, SizeOf(RING))) { Store(0, TAIL) }
        }
        Release(RTMX)
        Return(Local0)
    }
    // COUN: Used by ACPIDebug.kext to determine number of items in RING
    Method (COUN, 0, NotSerialized)
    {
        Acquire(RTMX, 0xFFFF)
        // return count of items in RING
        Subtract(HEAD, TAIL, Local0)
        if (LLess(Local0, 0)) { Add(Local0, SizeOf(RING), Local0) }
        Release(RTMX)
        Return(Local0)
    }
    // Helper functions for multiple params at one time
    Method (P1, 1, NotSerialized) { PUSH(Arg0) }
    Method (P2, 2, Serialized)
    {
        Name (TEMP, Package(2) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        PUSH(TEMP)
    }
    Method (P3, 3, Serialized)
    {
        Name (TEMP, Package(3) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        Store(Arg2, Index(TEMP, 2))
        PUSH(TEMP)
    }
    Method (P4, 4, Serialized)
    {
        Name (TEMP, Package(4) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        Store(Arg2, Index(TEMP, 2))
        Store(Arg3, Index(TEMP, 3))
        PUSH(TEMP)
    }
    Method (P5, 5, Serialized)
    {
        Name (TEMP, Package(5) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        Store(Arg2, Index(TEMP, 2))
        Store(Arg3, Index(TEMP, 3))
        Store(Arg4, Index(TEMP, 4))
        PUSH(TEMP)
    }
    Method (P6, 6, Serialized)
    {
        Name (TEMP, Package(6) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        Store(Arg2, Index(TEMP, 2))
        Store(Arg3, Index(TEMP, 3))
        Store(Arg4, Index(TEMP, 4))
        Store(Arg5, Index(TEMP, 5))
        PUSH(TEMP)
    }
    Method (P7, 7, Serialized)
    {
        Name (TEMP, Package(7) { })
        Store(Arg0, Index(TEMP, 0))
        Store(Arg1, Index(TEMP, 1))
        Store(Arg2, Index(TEMP, 2))
        Store(Arg3, Index(TEMP, 3))
        Store(Arg4, Index(TEMP, 4))
        Store(Arg5, Index(TEMP, 5))
        Store(Arg6, Index(TEMP, 6))
        PUSH(TEMP)
    }
    }
}
