// https://www.tonymacx86.com/threads/guide-creating-a-custom-ssdt-for-usbinjectall-kext.211311/
// Common port connector types are USB2 = 0, USB3 = 3, internal = 255.
// SSDT-UIAC-ALL.dsl
//
// This SSDT can be used as a template to build your own
// customization for USBInjectAll.kext.
//
// This SSDT contains all ports, so using it is the same as without
// a custom SSDT.  Delete ports that are not connected or ports you
// do not need.
//
// Change the UsbConnector or portType as needed to match your
// actual USB configuration.
//
// Note:
// portType=0 seems to indicate normal external USB2 port (as seen in MacBookPro8,1)
// portType=2 seems to indicate "internal device" (as seen in MacBookPro8,1)
// portType=4 is used by MacBookPro8,3 (reason/purpose unknown)
//
// ASUS UX430UA
// USB ports:

// USB-C(left side):
//	HS04 -> pendrive USB2
//	SS04 -> pendrive USB3
//	HS04 -> logitech 2.4G reciver
//	HS04 -> monitor hdmi->usb-c
//	
// USB3(left side):
//	HS01 -> pendrive USB2
//	SS01 -> pendrive USB3
//	HS01 -> logitech 2.4G reciver
//	
// USB2(right side):
//	HS02 -> pendrive USB2
//	HS02 -> pendrive USB3
//	HS02 -> logitech 2.4G reciver
// 
// CARD READER 
//	HS05 -> cards
//	
// Internal devices:	
// HS08: 
//	Broadcom wifi+bt
// HS06: 
//	USB2.0 HD UVC WebCam
// HS09:
//	ELAN:Fingerprint
// 
// so it uses: 
//    HS01, HS02, HS04, HS05, HS06, HS08, HS09
//    SS01, SS04
//
// !!! HS09 is for ELAN:fingerprint It doesn;t work so we want to disable it


DefinitionBlock ("", "SSDT", 2, "hack", "UIAC-ALL", 0)
{
    Device(UIAC)
    {
        Name(_HID, "UIA00000")
        Name(RMCF, Package()
        {
            
            
            "8086_9d2f", Package()
            {
                "port-count", Buffer() { 18, 0, 0, 0 },
                "ports", Package()
                {
                    "HS01", Package()
                    {
                        "UsbConnector", 0,
                        "port", Buffer() { 1, 0, 0, 0 },
                    },
                    "HS02", Package()
                    {
                        "UsbConnector", 0,
                        "port", Buffer() { 2, 0, 0, 0 },
                    },
                    "HS04", Package()
                    {
                        "UsbConnector", 3,
                        "port", Buffer() { 4, 0, 0, 0 },
                    },
                    "HS05", Package()
                    {
                        "UsbConnector", 0,
                        "port", Buffer() { 5, 0, 0, 0 },
                    },
                    "HS06", Package()
                    {
                        "UsbConnector", 255,
                        "port", Buffer() { 6, 0, 0, 0 },
                    },
                    "HS08", Package()
                    {
                        "UsbConnector", 255,
                        "port", Buffer() { 8, 0, 0, 0 },
                    },
                    "SS01", Package()
                    {
                        "UsbConnector", 3,
                        "port", Buffer() { 13, 0, 0, 0 },
                    },
                    "SS04", Package()
                    {
                        "UsbConnector", 3,
                        "port", Buffer() { 16, 0, 0, 0 },
                    },
                },
            },
        })
    }
}